{
    open Lexing
    open Myparser

    exception SyntaxError of string

  let next_line lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- { pos with pos_bol = lexbuf.lex_curr_pos;
                                    pos_lnum = pos.pos_lnum + 1 }
}

(* The textual form of an integer is a sequence of digits.
   Every sequence of digits represents an integer. *)
let intPattern = '-'? ['0'-'9']+
        (* ^ Pattern of ASCII characters. *)
        (*   Language of patterns / Regular expression *)
        (*   Any of the characters, '0', '1', '2', '3', ..., '9',
             Any sequence of one or more of these characters.
             "0", "123", "456"
             + ====== "Sequence of one or more" / "Non-empty sequence" / ... *)

let white = [' ' '\t']+
let newline = '\n'

rule read =
  parse
  | white { read lexbuf }
  | newline { next_line lexbuf; read lexbuf }
  | intPattern { INT(int_of_string(Lexing.lexeme lexbuf)) }
  | '+' { PLUS }
  | '-' { MINUS }
  | '(' { LPAREN }
  | ')' { RPAREN }
  | eof { EOF }
  | _ { raise (SyntaxError("Unexpected char: " ^ Lexing.lexeme lexbuf)) }

(* "45 - 32" ===> "[INT(45); MINUS; INT(32)]"
"         45        \n\n      -32" *)